#!/usr/bin/python3
#-*- coding: iso-8859-15 -*-

#############################################################################
#                                                                           #
#                            prueba_alarmas.py                              #
#                                                                           #
# Programa para configurar las dos alarmas y recibir su interrupcin con    #
# un mensaje por pantalla.                                                  #
# Al salir del programa (Ctrl+C) se limpian los GPIO, se desactivan las     #
# alarmas en caso de que no se haya recibido y se borra el flag en caso de  #
# que s se haya recibido.                                                  #
#                                                                           #
#############################################################################

# Importacin de las libreras
import RPi.GPIO as GPIO
import DS3231

RTC = DS3231.DS3231()

# Se configuran los pines en modo BCM (Broadcom)
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
# Configuracin del pin 17 como entrada con resistencia pull-up
GPIO.setup(17, GPIO.IN, pull_up_down=GPIO.PUD_UP)

# Rutina de tratamiento de la interrupcin
def interrupcion(channel):
    datos = RTC.leer_todo()
    print("\n\tALARMA RECIBIDA A LAS: " +
          "%02d:%02d:%02d" % (datos[0], datos[1], datos[2]))
    RTC.borrar_flags()

# Declaracin de la interrupcin:
# Pin, tipo de interrupcin y rutina de tratamiento asociada
GPIO.add_event_detect(17, GPIO.FALLING, callback=interrupcion, bouncetime=300)

# Declaracin de la alarma
print("\n\tPROGRAMACION DE ALARMA 1")
modo_1 = input("\nIntroduzca modo de alarma: ")
hora_1 = input("Introduzca hora (formato 24h): ")
minutos_1 = input("Introduzca minutos (0-59): ")
if modo_1 == 1:
    dia_1 = None
elif modo_1 == 2:
    dia_1 = input("Introduzca dia del mes: ")
elif modo_1 == 3:
    dia_1 = input("Introduzca dia de la semana (1-7): ")
else:
    raise ValueError ("El modo seleccionado no es correcto")

print("\n\tPROGRAMACION DE ALARMA 2")
modo_2 = input("\nIntroduzca modo de alarma: ")
hora_2 = input("Introduzca hora (formato 24h): ")
minutos_2 = input("Introduzca minutos (0-59): ")
if modo_2 == 1:
    dia_2 = None
elif modo_2 == 2:
    dia_2 = input("Introduzca dia del mes: ")
elif modo_2 == 3:
    dia_2 = input("Introduzca dia de la semana (1-7): ")
else:
    raise ValueError ("El modo seleccionado no es correcto")

# Se crea y activan las alarmas y se muestra mensaje de confirmacin por
# pantalla para comprobar la lectura de las mismas
RTC.crear_alarma_1(modo_1, hora_1, minutos_1, dia_1)
RTC.activar_alarma_1()
alarma_1 = RTC.leer_alarma_1()
if alarma_1[3] == None:
    print("\nAlarma 1 configurada en modo %d a las %02d:%02d"
          % (alarma_1[0], alarma_1[1], alarma_1[2]))
else:
    print("\nAlarma 1 configurada en modo %d a las %02d:%02d del dia %02d"
          % (alarma_1[0], alarma_1[1], alarma_1[2], alarma_1[3]))
      
RTC.crear_alarma_2(modo_2, hora_2, minutos_2, dia_2)
RTC.activar_alarma_2()
alarma_2 = RTC.leer_alarma_2()
if alarma_2[3] == None:
    print("\nAlarma 2 configurada en modo %d a las %02d:%02d"
          % (alarma_2[0], alarma_2[1], alarma_2[2]))
else:
    print("\nAlarma 2 configurada en modo %d a las %02d:%02d del dia %02d"
          % (alarma_2[0], alarma_2[1], alarma_2[2], alarma_2[3]))

# Ejecucin del programa
try:
    # Espera a recibir la interrupcin
    while(True):
        pass

# El programa se ejecuta hasta que se manda la seal de finalizacin
# desde el teclado
except KeyboardInterrupt:
    GPIO.cleanup()
    RTC.desactivar_alarma_1()
    RTC.desactivar_alarma_2()
    RTC.borrar_flags()
