#!/usr/bin/python3
#-*- coding: iso-8859-15 -*-

#############################################################################
#                                                                           #
#                             control_riego.py                              #
#                                                                           #
# Programa para configurar las dos alarmas por da de las que dispone el    #
# DS3231. Cuando se produzca la interrupcin de estas se encender un LED   #
# para emular la activacin de una bomba de riego. El LED (bomba de riego)  #
# permanecer encendido durante el tiempo especificado por el usuario.      #
#                                                                           #
# Dispone de un men con las distintas opciones que puede realizar el       #
# usuario como pueden ser programar las alarmas y ver el estado de estas,   #
# as como ver los datos de hora y fecha actuales y modificarlos.           #
#                                                                           #
# Para comprobar el funcionamiento se genera un fichero de texto en el      #
# que se van registrando todas las acciones que realiza el programa con     #
# los datos de hora y fecha a los que se producen.                          #
#                                                                           #
#############################################################################

# Importacin de las libreras
import RPi.GPIO as GPIO
import os
import sys
from time import sleep

# Se crea la ruta del mdulo DS3231 para importarlo
sys.path.append('/home/pi/TFG')
import DS3231

RTC = DS3231.DS3231()

# Desactiva alarmas y borra flags para evitar errores
RTC.desactivar_alarma_1()
RTC.desactivar_alarma_2()
RTC.borrar_flags()

# Obtiene hora actual para escribirla en el fichero
datos = RTC.leer_todo()
# Crea un fichero de texto si no existe y escribe la cabecera
fich = open('registro_riego.txt', 'w')
fich.write("\n\tREGISTRO DE EJECUCION")
fich.write("\n\n[%02d:%02d:%02d - "
            % (datos[0], datos[1], datos[2]) +
            "%02d/%02d/%02d]: "
            % (datos[4], datos[5], datos[6]) +
            "Inicio de ejecucion del programa")
fich.close()

# Se configuran los pines en modo BCM (Broadcom)
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
# Configuracion del pin 17 como entrada con resistencia pull-up
GPIO.setup(17, GPIO.IN, pull_up_down=GPIO.PUD_UP)
# Configuracion de los pines de salida
GPIO.setup(21, GPIO.OUT)

# Inicializacion de los LEDs (apagados)
GPIO.output(21, False)
estado = False
duracion = 0

# Funcin para aadir al fichero de texto los distintos eventos
def crear_evento(evento):
    if evento == 1:
        alarma_1 = RTC.leer_alarma_1()
        if alarma_1[3] == None:
            mensaje = ("Alarma 1 creada en modo %d a las %02d:%02d"
                      % (alarma_1[0], alarma_1[1], alarma_1[2]))
        else:
            mensaje = ("Alarma 1 creada en modo %d a las %02d:%02d "
                      % (alarma_1[0], alarma_1[1], alarma_1[2])
                       + "del dia %02d" % (alarma_1[3]))
    elif evento == 2:
        alarma_2 = RTC.leer_alarma_2()
        if alarma_2[3] == None:
            mensaje = ("Alarma 2 creada en modo %d a las %02d:%02d"
                      % (alarma_2[0], alarma_2[1], alarma_2[2]))
        else:
            mensaje = ("Alarma 2 creada en modo %d a las %02d:%02d "
                      % (alarma_2[0], alarma_2[1], alarma_2[2])
                       + "del dia %02d" % (alarma_2[3]))
    elif evento == 3:
        mensaje = "Mostrar estado alarmas"
    elif evento == 4:
        mensaje = "Alarma 1 desactivada"
    elif evento == 5:
        mensaje = "Alarma 2 desactivada"
    elif evento == 6:
        mensaje = "Mostrar datos de hora y fecha"
    elif evento == 7:
        mensaje = "Actualizacion de datos de hora y fecha"
    elif evento == 8:
        mensaje = "Fin de ejecucion del programa"
    elif evento == 9:
        mensaje = "RIEGO ACTIVADO"
    elif evento == 10:
        mensaje = "RIEGO DESACTIVADO"

    # Almacena la hora a la que se ha producido el evento
    datos = RTC.leer_todo()
    # Abre fichero existe sin sobreescribir informacin
    fich = open('registro_riego.txt', 'a')
    fich.write("\n\n[%02d:%02d:%02d - "
                % (datos[0], datos[1], datos[2]) +
                "%02d/%02d/%02d]: "
                % (datos[4], datos[5], datos[6]) +
                mensaje)
    fich.close()

# Rutina de tratamiento de la interrupcion
def interrupcion(channel):
    global estado
    global duracion
    global modo_1, hora_1, minutos_1, dia_1
    global modo_2, hora_2, minutos_2, dia_2
    
    if estado == False:
        estado = True
        crear_evento(9)
        GPIO.output(21, estado) # Enciende el LED de alarma

        # Programa la siguiente alarma con la que se desactivar el riego
        alarma_1 = RTC.leer_alarma_1()
        alarma_2 = RTC.leer_alarma_2()

        h = duracion / 60
        m = duracion % 60

        hora_sig_1 = alarma_1[1] + h
        minutos_sig_1 = alarma_1[2] + m
        hora_sig_2 = alarma_2[1] + h
        minutos_sig_2 = alarma_2[2] + m

        # Comprueba posible desbordamiento
        if minutos_sig_1 >= 60:
            hora_sig_1 += 1
            minutos_sig_1 -= 60

        if minutos_sig_2 >= 60:
            hora_sig_2 += 1
            minutos_sig_2 -= 60

        RTC.borrar_flags()
        RTC.crear_alarma_1(alarma_1[0], hora_sig_1, minutos_sig_1, alarma_1[3])
        RTC.crear_alarma_2(alarma_2[0], hora_sig_2, minutos_sig_2, alarma_2[3])
    elif estado == True:
        estado = False
        crear_evento(10)
        GPIO.output(21, estado) # Apaga el LED de alarma

        # Vuelve a configurar la alarma inicial sleccionada por el usuario
        RTC.borrar_flags()
        RTC.crear_alarma_1(modo_1, hora_1, minutos_1, dia_1)
        RTC.crear_alarma_2(modo_2, hora_2, minutos_2, dia_2)

# Declaracion de la interrupcion:
# Pin, tipo de interrupcion y rutina de tratamiento asociada
GPIO.add_event_detect(17, GPIO.FALLING, callback=interrupcion, bouncetime=300)

salir = False

# Ejecucin del programa
try:
    while not salir:
        # Limpia pantalla y muestra el men
        os.system('clear')
        print("")
        print("\t\t ____________________")
        print("\t\t|                    |")
        print("\t\t|  CONTROL DE RIEGO  |")
        print("\t\t|____________________|")

        print("")
        print("\tMENU")

        print("")
        print("\t1. Programar riego 1")
        print("\t2. Programar riego 2")
        print("\t3. Ver estado")
        print("\t4. Desactivar alarmas")
        print("\t5. Mostrar hora y fecha")
        print("\t6. Actualizar datos de hora y fecha")
        print("\t0. SALIR")
        print("")

        opcion = input("\tOpcion: ")

        # SELECCIN DE OPCIN
        
####### 1 - CREAR ALARMA 1
        if opcion == 1:
            os.system('clear')
            
            print("\n\n\n\t\tCONFIGURACION DE LA ALARMA 1")
            print("\n\tModos de funcionamiento:")
            print("\tModo 1: Hora y minutos seleccionados")
            print("\tModo 2: Hora, minutos y dia del mes")
            print("\tModo 3: Hora, minutos y dia de la semana")
            modo_1 = input("\n\tIntroduzca modo de alarma: ")
            hora_1 = input("\tIntroduzca hora (formato 24h): ")
            minutos_1 = input("\tIntroduzca minutos (0-59): ")
            if modo_1 == 1:
                dia_1 = None
            elif modo_1 == 2:
                dia_1 = input("\tIntroduzca dia del mes: ")
            elif modo_1 == 3:
                dia_1 = input("\tIntroduzca dia de la semana (1-7): ")
            else:
                raise ValueError ("El modo seleccionado no es correcto")

            duracion = input("\n\tIntroduzca duracion de " +
                                    "riego (minutos): ")

            # Se crea y activa la alarma y se muestra mensaje de
            # confirmacin por pantalla
            RTC.crear_alarma_1(modo_1, hora_1, minutos_1, dia_1)
            RTC.activar_alarma_1()

            crear_evento(1)

            print("")
            print("\t------------------------------------------------------")
            
            alarma_1 = RTC.leer_alarma_1()
            if alarma_1[3] == None:
                print("\n\tAlarma 1 configurada en modo %d a las "
                      % (alarma_1[0])
                      + "%02d:%02d" % (alarma_1[1], alarma_1[2]))
            else:
                print("\n\tAlarma 1 configurada en modo %d a las "
                      % (alarma_1[0])
                      + "%02d:%02d del dia %02d"
                      % (alarma_1[1], alarma_1[2], alarma_1[3]))

            print("\n\n\n>>> Enter para continuar")
            raw_input("")

####### 2 - CREAR ALARMA 2
            
        if opcion == 2:
            os.system('clear')
            
            print("\n\n\n\t\tCONFIGURACION DE LA ALARMA 2")
            print("\n\tModos de funcionamiento:")
            print("\tModo 1: Hora y minutos seleccionados")
            print("\tModo 2: Hora, minutos y dia del mes")
            print("\tModo 3: Hora, minutos y dia de la semana")
            modo_2 = input("\n\tIntroduzca modo de alarma: ")
            hora_2 = input("\tIntroduzca hora (formato 24h): ")
            minutos_2 = input("\tIntroduzca minutos (0-59): ")
            if modo_2 == 1:
                dia_2 = None
            elif modo_2 == 2:
                dia_2 = input("\tIntroduzca dia del mes: ")
            elif modo_2 == 3:
                dia_2 = input("\tIntroduzca dia de la semana (1-7): ")
            else:
                raise ValueError ("El modo seleccionado no es correcto")

            duracion = input("\n\tIntroduzca duracion de " +
                                    "riego (minutos): ")

            # Se crea y activa la alarma y se muestra mensaje de
            # confirmacin por pantalla
            RTC.crear_alarma_2(modo_2, hora_2, minutos_2, dia_2)
            RTC.activar_alarma_2()
            
            crear_evento(2)

            print("")
            print("\t------------------------------------------------------")
            
            alarma_2 = RTC.leer_alarma_2()
            if alarma_2[3] == None:
                print("\n\tAlarma 2 configurada en modo %d a las "
                      % (alarma_2[0])
                      + "%02d:%02d" % (alarma_2[1], alarma_2[2]))
            else:
                print("\n\tAlarma 2 configurada en modo %d a las "
                      % (alarma_2[0])
                      + "%02d:%02d del dia %02d"
                      % (alarma_2[1], alarma_2[2], alarma_2[3]))

            print("\n\n\n>>> Enter para continuar")
            raw_input("")

####### 3 - VER ESTADO
            
        if opcion == 3:
            os.system('clear')

            alarma_1 = RTC.leer_alarma_1()
            alarma_2 = RTC.leer_alarma_2()
            
            print("\n\n\n\t\tESTADO DE LAS ALARMAS")

            if ((RTC.leer(0x0e) & 0b1) and (RTC.leer(0x0e) & 0b10)):
                print("\n\tAlarma 1: ACTIVADA")
                print("\n\tAlarma 2: ACTIVADA")

                print("")
                print("\t------------------------------------------------------")

                if alarma_1[3] == None:
                    print("\n\tAlarma 1 configurada en modo %d a las "
                          % (alarma_1[0])
                          + "%02d:%02d" % (alarma_1[1], alarma_1[2]))
                else:
                    print("\n\tAlarma 1 configurada en modo %d a las "
                          % (alarma_1[0])
                          + "%02d:%02d del dia %02d"
                          % (alarma_1[1], alarma_1[2], alarma_1[3]))

                if alarma_2[3] == None:
                    print("\n\tAlarma 2 configurada en modo %d a las "
                          % (alarma_2[0])
                          + "%02d:%02d" % (alarma_2[1], alarma_2[2]))
                else:
                    print("\n\tAlarma 2 configurada en modo %d a las "
                          % (alarma_2[0])
                          + "%02d:%02d del dia %02d"
                          % (alarma_2[1], alarma_2[2], alarma_2[3]))
            elif (RTC.leer(0x0e) & 0b1):
                print("\n\tAlarma 1: ACTIVADA")
                print("\n\tAlarma 2: DESACTIVADA")

                print("")
                print("\t------------------------------------------------------")

                if alarma_1[3] == None:
                    print("\n\tAlarma 1 configurada en modo %d a las "
                          % (alarma_1[0])
                          + "%02d:%02d" % (alarma_1[1], alarma_1[2]))
                else:
                    print("\n\tAlarma 1 configurada en modo %d a las "
                          % (alarma_1[0])
                          + "%02d:%02d del dia %02d"
                          % (alarma_1[1], alarma_1[2], alarma_1[3]))
            elif (RTC.leer(0x0e) & 0b10):
                print("\n\tAlarma 1: DESACTIVADA")
                print("\n\tAlarma 2: ACTIVADA")

                print("")
                print("\t------------------------------------------------------")

                if alarma_2[3] == None:
                    print("\n\tAlarma 2 configurada en modo %d a las "
                          % (alarma_2[0])
                          + "%02d:%02d" % (alarma_2[1], alarma_2[2]))
                else:
                    print("\n\tAlarma 2 configurada en modo %d a las "
                          % (alarma_2[0])
                          + "%02d:%02d del dia %02d"
                          % (alarma_2[1], alarma_2[2], alarma_2[3]))
            else:
                print("\n\tAlarma 1: DESACTIVADA")
                print("\n\tAlarma 2: DESACTIVADA")

            if estado == False:
                mensaje_estado = "INACTIVO"
            elif estado == True:
                mensaje_estado = "ACTIVO"

            print("")
            print("\t------------------------------------------------------")
            print("\n\t\tESTADO DEL RIEGO: " + mensaje_estado)
            if duracion is not 0:
                print("\t\tDuracion de riego: " + str(duracion) + " minutos")

            crear_evento(3)

            print("\n\n\n>>> Enter para continuar")
            raw_input("")

####### 4 - DESACTIVAR ALARMA
            
        if opcion == 4:
            os.system('clear')

            print("\n\n\n\tSeleccione la alarma que desee desactivar:")
            print("")
            print("\t1. Alarma 1")
            print("\t2. Alarma 2")
            print("\t0. Volver")
            print("")
            
            opcion_2 = input("\tOpcion: ")

            if opcion_2 == 1:
                RTC.desactivar_alarma_1()

                crear_evento(4)

                print("\n\tALARMA 1 DESACTIVADA")

                print("\n\n\n>>> Enter para continuar")
                raw_input("")

            if opcion_2 == 2:
                RTC.desactivar_alarma_2()

                crear_evento(5)

                print("\n\tALARMA 2 DESACTIVADA")

                print("\n\n\n>>> Enter para continuar")
                raw_input("")

            if opcion_2 == 0:
                pass

####### 5 - MOSTRAR HORA Y FECHA
        
        if opcion == 5:
            os.system('clear')
            
            # Obtiene el vector con todos los datos
            datos = RTC.leer_todo()

            # Determina el da de la semana
            if datos[3] == 1:
                dia_semana = "Lunes"
            elif datos[3] == 2:
                dia_semana = "Martes"
            elif datos[3] == 3:
                dia_semana = "Miercoles"
            elif datos[3] == 4:
                dia_semana = "Jueves"
            elif datos[3] == 5:
                dia_semana = "Viernes"
            elif datos[3] == 6:
                dia_semana = "Sabado"
            elif datos[3] == 7:
                dia_semana = "Domingo"
                
            # Calcula el ao con 4 dgitos a partir del siglo definido y la
            # lectura de ao
            anio = (DS3231.SIGLO - 1)*100 + datos[6]
            
            # Muestra la hora por pantalla
            print("\n\n\n\tHora actual: %02d:%02d"% (datos[0], datos[1]))

            # Muestra la fecha por pantalla
            print("\n\tFecha: " + dia_semana + " %02d/%02d/%04d"
                  % (datos[4], datos[5], anio))

            crear_evento(6)

            print("\n\n\n>>> Enter para continuar")
            raw_input("")

####### 6 - ACTUALIZACIN DE DATOS
            
        if opcion == 6:
            os.system('clear')

            print("\n\n\n\tSeleccione modo de actualizacion:")
            print("")
            print("\t1. Automatico")
            print("\t2. Manual")
            print("\t0. Volver")
            print("")
            
            opcion_2 = input("\tOpcion: ")

            if opcion_2 == 1:
                RTC.actualizar_fecha()

                crear_evento(7)

                print("\n\n\n\tDATOS DE HORA Y FECHA ACTUALIZADOS")

                print("\n\n\n>>> Enter para continuar")
                raw_input("")

            if opcion_2 == 2:
                os.system('clear')
                print("\n\n\n\tACTUALIZACION MANUAL DE DATOS DE HORA Y FECHA")
                print("\tSi no se desea modificar algun dato, introducir None")

                # Se solicitan los datos por pantalla
                hora = input("\n\tIntroduzca la hora (formato 24h): ")
                minutos = input("\tIntroduzca los minutos: ")
                dia_semana = input("\tIntroduzca el dia de la semana (1-7): ")
                dia_mes = input("\tIntroduzca el dia del mes: ")
                mes = input("\tIntroduzca mes (1-12): ")
                anio = input("\tIntroduzca anio (aa): ")


                # (hora, minutos, segundos, dia_semana, dia_mes, mes, anio(aa))
                RTC.escribir_fecha(hora,minutos,0,dia_semana,dia_mes,mes,anio)

                crear_evento(7)
                
                print("\n\n\n\tDATOS DE HORA Y FECHA ACTUALIZADOS")

                print("\n\n\n>>> Enter para continuar")
                raw_input("")

            if opcion_2 == 0:
                pass

####### 0 - SALIR
            
        if opcion == 0:
            os.system('clear')

            print("\n\n\n\tLas alarmas activas se desactivaran.")
            salida = raw_input("\n\tSeguro que desea salir? (s/n): ")

            if salida == 's':
                GPIO.cleanup()
                RTC.desactivar_alarma_1()
                RTC.desactivar_alarma_2()
                RTC.borrar_flags()
                salir = True
                os.system('clear')

                crear_evento(8)
            elif salida == 'n':
                pass

# El programa se ejecuta hasta que se manda la senial de finalizacion
# desde el teclado, y es ahi cuando se liberan los puertos, se apagan
# los LEDs y se desactiva la alarma
except KeyboardInterrupt:
    GPIO.output(21, False)
    GPIO.cleanup()
    RTC.desactivar_alarma_1()
    RTC.desactivar_alarma_2()
    RTC.borrar_flags()
