#!/usr/bin/python3
#-*- coding: iso-8859-15 -*-

#############################################################################
#                                                                           #
#                              subrutina.py                                 #
#                                                                           #
# Alarma peridica cada hora para realizar una lectura del sensor DHT22     #
# para obtener datos de temperatura y humedad que se almacenan en un        #
# fichero de texto con la informacin de la hora y fecha a la que fueron    #
# obtenidos.                                                                #
#                                                                           #
#############################################################################

# Importacin de las libreras
import RPi.GPIO as GPIO
from time import sleep
import sys
import Adafruit_DHT as DHT

# Se crea la ruta del mdulo DS3231 para exportarlo
sys.path.append('/home/pi/TFG')
import DS3231

RTC = DS3231.DS3231()

# Desactiva alarmas y borra flags para evitar errores
RTC.desactivar_alarma_1()
RTC.desactivar_alarma_2()
RTC.borrar_flags()

def lectura_sensor():
    datos = RTC.leer_todo()
    
    # Lectura del sensor
    h,t =  DHT.read_retry(DHT.DHT22, 26)

    # Mensaje por terminal
    print("\n[%02d:%02d:%02d - "
           % (datos[0], datos[1], datos[2]) +
           "%02d/%02d/%02d]: "
           % (datos[4], datos[5], datos[6]) +
           "Lectura realizada")
    print("Temperatura: %.01f" % t + " *C // " +
          "Humedad: %.01f" % h + " %")

    # Escritura de datos en fichero de texto sin sobreescribir informacin
    fich = open('registro_ejecucion.txt', 'a')
    fich.write("\n\n[%02d:%02d:%02d - "
                % (datos[0], datos[1], datos[2]) +
                "%02d/%02d/%02d] -> "
                % (datos[4], datos[5], datos[6]) +
                "Temperatura: %.01f" % t + " *C // " +
                "Humedad: %.01f" % h + " %")
    fich.close()

# Obtiene hora actual para escribirla en el fichero
datos = RTC.leer_todo()
# Crea un fichero de texto si no existe y escribe la cabecera
fich = open('registro_ejecucion.txt', 'w')
fich.write("\n\tREGISTRO DE EJECUCION")
fich.write("\n\n[%02d:%02d:%02d - "
            % (datos[0], datos[1], datos[2]) +
            "%02d/%02d/%02d]: "
            % (datos[4], datos[5], datos[6]) +
            "Inicio de ejecucion del programa")
fich.close()

# Realiza lectura con la ejecucin del programa
lectura_sensor()

# Se configuran los pines en modo BCM (Broadcom)
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
# Configuracin del pin 17 como entrada con resistencia pull-up
GPIO.setup(17, GPIO.IN, pull_up_down=GPIO.PUD_UP)

# Rutina de tratamiento de la interrupcin
def interrupcion(channel):
    lectura_sensor()

    # Se programa la alarma a la hora siguiente
    RTC.borrar_flags()
    alarma_1 = RTC.leer_alarma_1()

    if alarma_1[1] == 23:
        hora_sig = 0
    else:
        hora_sig = alarma_1[1] + 1
    
    RTC.crear_alarma_1(1, hora_sig, alarma_1[2])

# Declaracin de la interrupcin:
# Pin, tipo de interrupcin y rutina de tratamiento asociada
GPIO.add_event_detect(17, GPIO.FALLING, callback=interrupcion, bouncetime=300)

# Se crea y activa la alarma para la hora siguiente de ejecucin del programa
RTC.crear_alarma_1(1, datos[0] + 1, datos[1])
RTC.activar_alarma_1()

# Ejecucin del programa
try:
    # Espera a recibir la interrupcin
    while(True):
        pass

# El programa se ejecuta hasta que se manda la seal de finalizacin
# desde el teclado
except KeyboardInterrupt:
    GPIO.cleanup()
    RTC.desactivar_alarma_1()
    RTC.borrar_flags()
